/*DIODES Inc. EVB Information------------------------------------------------------------------------------------
Device name：AL58263/Q
Interface：4-Wire Serial (LAT, DI, DO, DCK)
Development Platform：Arduino UNO , MCU：ATmega328PB
Function Achievement：Scrolling Mode , LED Open/short Circuit Detection Mode , LED Smart Detection Mode
                      Sleep Mode , Rext Short Circuit Detection Mode

Revision Record：
2023.10.09. Original version A06
2024.06.14. Optimization function switch , version A07
----------------------------------------------------------------------------------------------------------------*/ 
#define NunberOfICs 2
#define ICDataBits 16
#define TimeDelay1 1000
#define TimeDelay2 40
#define TimeDelay3 10
#define DisplayBrightness 12345  //Define the brightness of the status display LED (16 LEDs on the lower left side) (Brightness MAX : 65535)

#define Dat0 0                             //Define Dat0 at Arduino digital pin 0
#define Dck0 1                             //Define Dck0 at Arduino digital pin 1
#define Lat0 2                             //Define Lat0 at Arduino digital pin 2
#define Dat2 3                             //Define Dat2 at Arduino digital pin 3
#define GckMode0 4                         //Define GckMode2 at Arduino digital pin 4
#define Gck0 5                             //Define Gck0 at Arduino digital pin 5
#define CompulsoryLEDOpenDetectionKey1 10  //Define the trigger button for forced LED open circuit detection at Arduino digital pin 10
#define CompulsoryLEDShortDetectionKey1 9  //Define the trigger button for forced LED Short circuit detection at Arduino digital pin 9
#define CompulsoryLEDSmartDetectionKey1 8  //Define the trigger button for forced LED Smart circuit detection at Arduino digital pin 8
#define CompulsorySleepModeKey1 7          //Define the trigger button for forced Sleep at Arduino digital pin 7
#define CompulsoryREXTPinDetectionKey1 6   //Define the trigger button for forced  Compulsory REXT Pin Detection at Arduino digital pin 6

#define VinSensorPin A0

//-Power startup initialization---------------------------------------------
void setup() {
  pinMode(Dat0, OUTPUT);
  pinMode(Dck0, OUTPUT);
  pinMode(Lat0, OUTPUT);
  pinMode(Dat2, INPUT);
  pinMode(GckMode0, INPUT);
  pinMode(Gck0, INPUT);

  pinMode(CompulsoryLEDOpenDetectionKey1, INPUT);
  pinMode(CompulsoryLEDShortDetectionKey1, INPUT);
  pinMode(CompulsoryLEDSmartDetectionKey1, INPUT);
  pinMode(CompulsorySleepModeKey1, INPUT);
  pinMode(CompulsoryREXTPinDetectionKey1, INPUT);
  OCR0A = 2;  //Top value A ,PWM Frequency = PWM CLK / (OCR0A+1) = 16/3 = 5.33MHz
  OCR0B = 1;  //Toggle value B, Duty = (OCR0B + 1)/(OCR0A + 1) = 2/3 = 66.7%
  TCCR0A = 0;
  TCCR0A = _BV(COM0A0) | _BV(COM0B1) | _BV(WGM01) | _BV(WGM00);  //Set Timer0 to varying top limit fast PWM mode
  TCCR0B = 0;
  TCCR0B = _BV(WGM02) | _BV(WGM01) | _BV(WGM00) | _BV(CS00);  //another way to set prescaler CS2= 1 , PWM CLK = 16MHz / 1 =16MHZ
}

//----------------------------------------------
void delay001(void) {

  for (long i = 0; i < TimeDelay2; i++) {
    delay(TimeDelay1);
  }
}

void delay002(void) {
  for (long i = 0; i < TimeDelay3; i++) {
    delay(TimeDelay1);
  }
}

//----------------------------------------------
void Dat0_Hi(void) {
  digitalWrite(Dat0, HIGH);
}
void Dat0_Lo(void) {
  digitalWrite(Dat0, LOW);
}
void Dck0_Hi(void) {
  digitalWrite(Dck0, HIGH);
}
void Dck0_Lo(void) {
  digitalWrite(Dck0, LOW);
}
void Lat0_Hi(void) {
  digitalWrite(Lat0, HIGH);
}
void Lat0_Lo(void) {
  digitalWrite(Lat0, LOW);
}
void Gck0_Hi(void) {
  digitalWrite(Lat0, HIGH);
}
void Gck0_Lo(void) {
  digitalWrite(Lat0, LOW);
}
//----------------------------------------------
void SHiftDataOut(unsigned int y0) {
  Dat0_Lo();
  if (bitRead(y0, 15)) Dat0_Hi();
  else Dat0_Lo();
  Dck0_Hi();
  Dck0_Lo();

  if (bitRead(y0, 14)) Dat0_Hi();
  else Dat0_Lo();
  Dck0_Hi();
  Dck0_Lo();

  if (bitRead(y0, 13)) Dat0_Hi();
  else Dat0_Lo();
  Dck0_Hi();
  Dck0_Lo();

  if (bitRead(y0, 12)) Dat0_Hi();
  else Dat0_Lo();
  Dck0_Hi();
  Dck0_Lo();

  if (bitRead(y0, 11)) Dat0_Hi();
  else Dat0_Lo();
  Dck0_Hi();
  Dck0_Lo();

  if (bitRead(y0, 10)) Dat0_Hi();
  else Dat0_Lo();
  Dck0_Hi();
  Dck0_Lo();

  if (bitRead(y0, 9)) Dat0_Hi();
  else Dat0_Lo();
  Dck0_Hi();
  Dck0_Lo();

  if (bitRead(y0, 8)) Dat0_Hi();
  else Dat0_Lo();
  Dck0_Hi();
  Dck0_Lo();

  if (bitRead(y0, 7)) Dat0_Hi();
  else Dat0_Lo();
  Dck0_Hi();
  Dck0_Lo();

  if (bitRead(y0, 6)) Dat0_Hi();
  else Dat0_Lo();
  Dck0_Hi();
  Dck0_Lo();

  if (bitRead(y0, 5)) Dat0_Hi();
  else Dat0_Lo();
  Dck0_Hi();
  Dck0_Lo();

  if (bitRead(y0, 4)) Dat0_Hi();
  else Dat0_Lo();
  Dck0_Hi();
  Dck0_Lo();

  if (bitRead(y0, 3)) Dat0_Hi();
  else Dat0_Lo();
  Dck0_Hi();
  Dck0_Lo();

  if (bitRead(y0, 2)) Dat0_Hi();
  else Dat0_Lo();
  Dck0_Hi();
  Dck0_Lo();

  if (bitRead(y0, 1)) Dat0_Hi();
  else Dat0_Lo();
  Dck0_Hi();
  Dck0_Lo();

  if (bitRead(y0, 0)) Dat0_Hi();
  else Dat0_Lo();
  Dck0_Hi();
  Dck0_Lo();
  Dat0_Lo();
}
//----------------------------------------------
void EndCMD(void) {
  Lat0_Lo();
  Dck0_Hi();
  Dck0_Lo();
}
//----------------------------------------------
void CmdDataLatch(void) {
  Dck0_Lo();
  Lat0_Hi();
  Lat0_Lo();
}
//----------------------------------------------
void CmdGlobalLatch(void) {
  int i;

  Lat0_Lo();
  for (int i = 0; i < 14; i++) {
    Dck0_Hi();
    Dck0_Lo();
  }

  Lat0_Hi();
  Dck0_Hi();
  Dck0_Lo();
  Dck0_Hi();
  Dck0_Lo();

  Lat0_Lo();
}
//----------------------------------------------
void CmdReadingImageDataAndCommandData(void) {
  int i;

  Lat0_Lo();
  for (int i = 0; i < 10; i++) {
    Dck0_Hi();
    Dck0_Lo();
  }

  Lat0_Hi();
  for (int i = 0; i < 6; i++) {
    Dck0_Hi();
    Dck0_Lo();
  }

  Lat0_Lo();
}
//----------------------------------------------
void CmdReturnToNormalMode(void) {
  int i;

  Lat0_Lo();
  for (int i = 0; i < 8; i++) {
    Dck0_Hi();
    Dck0_Lo();
  }

  Lat0_Hi();
  for (int i = 0; i < 8; i++) {
    Dck0_Hi();
    Dck0_Lo();
  }

  Lat0_Lo();
}
//----------------------------------------------
void CmdWritingCommand(unsigned int y0) {

  Lat0_Lo();
  Dat0_Lo();
  if (bitRead(y0, 15)) Dat0_Hi();
  else Dat0_Lo();
  Dck0_Hi();
  Dck0_Lo();

  if (bitRead(y0, 14)) Dat0_Hi();
  else Dat0_Lo();
  Dck0_Hi();
  Dck0_Lo();

  if (bitRead(y0, 13)) Dat0_Hi();
  else Dat0_Lo();
  Dck0_Hi();
  Dck0_Lo();

  if (bitRead(y0, 12)) Dat0_Hi();
  else Dat0_Lo();
  Dck0_Hi();
  Dck0_Lo();

  if (bitRead(y0, 11)) Dat0_Hi();
  else Dat0_Lo();
  Dck0_Hi();
  Dck0_Lo();

  if (bitRead(y0, 10)) Dat0_Hi();
  else Dat0_Lo();
  Dck0_Hi();
  Dck0_Lo();

  Lat0_Hi();
  if (bitRead(y0, 9)) Dat0_Hi();
  else Dat0_Lo();
  Dck0_Hi();
  Dck0_Lo();

  if (bitRead(y0, 8)) Dat0_Hi();
  else Dat0_Lo();
  Dck0_Hi();
  Dck0_Lo();

  if (bitRead(y0, 7)) Dat0_Hi();
  else Dat0_Lo();
  Dck0_Hi();
  Dck0_Lo();

  if (bitRead(y0, 6)) Dat0_Hi();
  else Dat0_Lo();
  Dck0_Hi();
  Dck0_Lo();

  if (bitRead(y0, 5)) Dat0_Hi();
  else Dat0_Lo();
  Dck0_Hi();
  Dck0_Lo();

  if (bitRead(y0, 4)) Dat0_Hi();
  else Dat0_Lo();
  Dck0_Hi();
  Dck0_Lo();

  if (bitRead(y0, 3)) Dat0_Hi();
  else Dat0_Lo();
  Dck0_Hi();
  Dck0_Lo();

  if (bitRead(y0, 2)) Dat0_Hi();
  else Dat0_Lo();
  Dck0_Hi();
  Dck0_Lo();

  if (bitRead(y0, 1)) Dat0_Hi();
  else Dat0_Lo();
  Dck0_Hi();
  Dck0_Lo();

  if (bitRead(y0, 0)) Dat0_Hi();
  else Dat0_Lo();
  Dck0_Hi();
  Dck0_Lo();
  Dat0_Lo();
  Lat0_Lo();
}
//----------------------------------------------
void CmdResetPwmCounter(void) {
  int i;

  Lat0_Lo();
  for (int i = 0; i < 4; i++) {
    Dck0_Hi();
    Dck0_Lo();
  }
  Lat0_Hi();

  for (int i = 0; i < 12; i++) {
    Dck0_Hi();
    Dck0_Lo();
  }
  Lat0_Lo();
}
//----------------------------------------------
void CleanCMDBuffer(void) {
  unsigned int CMD = 0, i;
  CmdResetPwmCounter();  //Reset IC CMD　Data

  Lat0_Lo();
  for (int i = 0; i < (unsigned int)(32); i++) {
    Dck0_Hi();
    Dck0_Lo();
  }
  CmdWritingCommand(0);
}
//----------------------------------------------
void InitICInnrGCK(void) {
  unsigned int CMD = 0;
  unsigned int Gain = 0x37;
  bitSet(CMD, 15);  //CMD[15],set 1, Data transmitting Format: 16 times of data latch + one global latch
  bitSet(CMD, 14);  //CMD[14],set 1, turn off (When the voltage of REXT is below 0.15V,all outputs will be forced off.)
  bitSet(CMD, 13);  //CMD[13],set 1, The GCK signal stems from an internal oscillator, 1.5MHz.
  //bitSet(CMD,12);  //CMD[12],set 1, PWM mode
  bitSet(CMD, 11);  //CMD[11],set 1, Execute counter reset by inserting 12 or 13 DCKs (rising edge) when LAT keeps at high
  bitSet(CMD, 10);  //CMD[10],set 1, When the device receives a latch signal,the new frame is updated immediately

  if (bitRead(Gain, 5)) bitSet(CMD, 9);  //CMD[9:4], 6bits DA data for global current control
  if (bitRead(Gain, 4)) bitSet(CMD, 8);
  if (bitRead(Gain, 3)) bitSet(CMD, 7);
  if (bitRead(Gain, 2)) bitSet(CMD, 6);
  if (bitRead(Gain, 1)) bitSet(CMD, 5);
  if (bitRead(Gain, 0)) bitSet(CMD, 4);

  bitSet(CMD, 3);  //CMD[3:2],set 0x11 , Threshold voltage for LED short Detection   (LED VF: 0x00=2V,0x01=3V,0x10=4V,0x11=4.5V)
  bitSet(CMD, 2);  //CMD[3:2],set 0x11 , Threshold voltage for LED short Detection   (LED VF: 0x00=2V,0x01=3V,0x10=4V,0x11=4.5V)

  //bitSet(CMD,1);  //CMD[1],set 0 , Do not turn on sleep energy-saving mode
  //bitSet(CMD,1);  //CMD[0],set 0 , Do not turn on the grayscale clock GCK watchdog function

  for (int i = 0; i < NunberOfICs; i++) { SHiftDataOut(CMD); }
  CmdWritingCommand(CMD);
}
//----------------------------------------------
void InitICExternalGCK(void) {
  unsigned int CMD = 0;
  unsigned int Gain = 0x37;
  bitSet(CMD, 15);  //CMD[15],set 1, Data transmitting Format: 16 times of data latch + one global latch
  bitSet(CMD, 14);  //CMD[14],set 1, turn off (When the voltage of REXT is below 0.15V,all outputs will be forced off.)
  //bitSet(CMD,13);  //CMD[13],set 1, The GCK signal stems from an internal oscillator, 1.5MHz.
  //bitSet(CMD,12);  //CMD[12],set 1, PWM mode
  bitSet(CMD, 11);  //CMD[11],set 1, Execute counter reset by inserting 12 or 13 DCKs (rising edge) when LAT keeps at high
  bitSet(CMD, 10);  //CMD[10],set 1, When the device receives a latch signal,the new frame is updated immediately

  if (bitRead(Gain, 5)) bitSet(CMD, 9);  //CMD[9:4], 6bits DA data for global current control
  if (bitRead(Gain, 4)) bitSet(CMD, 8);
  if (bitRead(Gain, 3)) bitSet(CMD, 7);
  if (bitRead(Gain, 2)) bitSet(CMD, 6);
  if (bitRead(Gain, 1)) bitSet(CMD, 5);
  if (bitRead(Gain, 0)) bitSet(CMD, 4);

  bitSet(CMD, 3);  //CMD[3:2],set 0x11 , Threshold voltage for LED short Detection   (LED VF: 0x00=2V,0x01=3V,0x10=4V,0x11=4.5V)
  bitSet(CMD, 2);  //CMD[3:2],set 0x11 , Threshold voltage for LED short Detection   (LED VF: 0x00=2V,0x01=3V,0x10=4V,0x11=4.5V)

  //bitSet(CMD,1);  //CMD[1],set 0 , Do not turn on sleep energy-saving mode
  //bitSet(CMD,1);  //CMD[0],set 0 , Do not turn on the grayscale clock GCK watchdog function

  for (int i = 0; i < NunberOfICs; i++) { SHiftDataOut(CMD); }
  CmdWritingCommand(CMD);
}
//----------------------------------------------
void EnableExternalGckOutput(void) {
  pinMode(Gck0, OUTPUT);
}
void DisableExternalGckOutput(void) {
  pinMode(Gck0, INPUT);
}
void GCKModeCheck(void) {
  DisableExternalGckOutput();
  pinMode(GckMode0, INPUT);
  if (digitalRead(GckMode0)) {
    InitICExternalGCK();  // The GCK signal stems from Arduino uno 3 , 5.3MHz.
    EnableExternalGckOutput();
  } else {
    DisableExternalGckOutput();
    InitICInnrGCK();  // The GCK signal stems from an internal oscillator, 1.5MHz.
  }
}
//----------------------------------------------
void LEDOutput(int buf1[ICDataBits], int buf2[ICDataBits]) {
  int i;
  GCKModeCheck();
  for (int i = 0; i < ICDataBits; i++) {
    SHiftDataOut(buf1[i]);
    SHiftDataOut(buf2[i]);
    CmdDataLatch();
    EndCMD();
  }
  CmdGlobalLatch();
  CmdResetPwmCounter();
}
//----------------------------------------------
void CleanDataBuffer(void) {
  unsigned int CMD = 0, i;
  int buf1[ICDataBits], buf2[ICDataBits];

  for (int i = 0; i < 16; i++) {
    buf1[i] = 0;
    buf2[i] = 0;
  }
  LEDOutput(buf1, buf2);
}
//----------------------------------------------
int CompulsoryLEDOpenDetection(void) {
  unsigned int Result1;

  Lat0_Hi();
  for (int i = 0; i < 14; i++) {
    Dck0_Hi();
    Dck0_Lo();
  }
  Dat0_Hi();
  Dat0_Lo();
  Dat0_Hi();
  Dat0_Lo();
  Lat0_Lo();
  Lat0_Lo();
  Result1 = 0;
  //if (digitalRead(Dat2)) { bitSet(Result1, 31); }
  for (int i = 32; i > 0; i--) {
    if (digitalRead(Dat2)) { bitSet(Result1, (i - 1)); }
    Dck0_Hi();
    Dck0_Lo();
  }
  Lat0_Lo();
  return Result1;
}
//----------------------------------------------
void DemoCompulsoryLEDOpenDetection(void) {
  int buf1[ICDataBits], buf2[ICDataBits];
  int ErrorState1;
  if (!digitalRead(CompulsoryLEDOpenDetectionKey1)) {
    ErrorState1 = CompulsoryLEDOpenDetection();
    for (int i = 0; i < 16; i++) {
      if (bitRead(ErrorState1, 15 - i)) buf1[i] = 0;
      else buf1[i] = DisplayBrightness;
      buf2[i] = 0;
    }
    LEDOutput(buf1, buf2);
    delay001();
    while (!digitalRead(CompulsoryLEDOpenDetectionKey1)) {}  //renew240614
    while (digitalRead(CompulsoryLEDOpenDetectionKey1)) {
      ErrorState1 = CompulsoryLEDOpenDetection();
      for (int i = 0; i < 16; i++) {
        if (bitRead(ErrorState1, 15 - i)) buf1[i] = 0;
        else buf1[i] = DisplayBrightness;
        buf2[i] = 0;
      }
      LEDOutput(buf1, buf2);
      delay(1000);
    }
  }
}
//----------------------------------------------
int CompulsoryLEDShortDetection(void) {
  unsigned int Result1;

  Lat0_Hi();
  for (int i = 0; i < 14; i++) {
    Dck0_Hi();
    Dck0_Lo();
  }
  for (int i = 0; i < 4; i++) {
    Dat0_Hi();
    Dat0_Lo();
  }
  Lat0_Lo();
  Lat0_Lo();
  Result1 = 0;
  if (digitalRead(Dat2)) { bitSet(Result1, 31); }
  for (int i = 31; i > 0; i--) {
    Dck0_Hi();
    if (digitalRead(Dat2)) { bitSet(Result1, (i - 1)); }
    Dck0_Lo();
  }
  Lat0_Lo();
  return Result1;
}
//----------------------------------------------
void DemoCompulsoryLEDShortDetection(void) {
  int buf1[ICDataBits], buf2[ICDataBits];
  int ErrorState1;
  if (!digitalRead(CompulsoryLEDShortDetectionKey1)) {
    ErrorState1 = CompulsoryLEDShortDetection();
    for (int i = 0; i < 16; i++) {
      if (bitRead(ErrorState1, 15 - i)) buf1[i] = 0;
      else buf1[i] = DisplayBrightness;
      buf2[i] = 0;
    }
    LEDOutput(buf1, buf2);
    delay001();
    while (!digitalRead(CompulsoryLEDShortDetectionKey1)) {}  //renew240614
    while (digitalRead(CompulsoryLEDShortDetectionKey1)) {
      ErrorState1 = CompulsoryLEDShortDetection();
      for (int i = 0; i < 16; i++) {
        if (bitRead(ErrorState1, 15 - i)) buf1[i] = 0;
        else buf1[i] = DisplayBrightness;
        buf2[i] = 0;
      }
      LEDOutput(buf1, buf2);
      delay(1000);
    }
  }
}
//----------------------------------------------
int CompulsoryLEDSmartDetection(void) {
  unsigned int Result1;

  Lat0_Hi();
  for (int i = 0; i < 14; i++) {
    Dck0_Hi();
    Dck0_Lo();
  }
  for (int i = 0; i < 10; i++) {
    Dat0_Hi();
    Dat0_Lo();
  }
  Lat0_Lo();
  Lat0_Lo();
  Result1 = 0;
  if (digitalRead(Dat2)) { bitSet(Result1, 31); }
  for (int i = 31; i > 0; i--) {
    Dck0_Hi();
    if (digitalRead(Dat2)) { bitSet(Result1, (i - 1)); }
    Dck0_Lo();
  }
  Lat0_Lo();
  return Result1;
}
//----------------------------------------------
void DemoCompulsoryLEDSmartDetection(void) {
  int buf1[ICDataBits], buf2[ICDataBits];
  int ErrorState1;
  if (!digitalRead(CompulsoryLEDSmartDetectionKey1)) {
    ErrorState1 = CompulsoryLEDSmartDetection();
    for (int i = 0; i < 16; i++) {
      if (bitRead(ErrorState1, 15 - i)) buf1[i] = 0;
      else buf1[i] = DisplayBrightness;
      buf2[i] = 0;
    }
    LEDOutput(buf1, buf2);
    delay001();
    while (!digitalRead(CompulsoryLEDSmartDetectionKey1)) {}  //renew240614
    while (digitalRead(CompulsoryLEDSmartDetectionKey1)) {
      ErrorState1 = CompulsoryLEDSmartDetection();
      for (int i = 0; i < 16; i++) {
        if (bitRead(ErrorState1, 15 - i)) buf1[i] = 0;
        else buf1[i] = DisplayBrightness;
        buf2[i] = 0;
      }
      LEDOutput(buf1, buf2);
      delay001();
    }
  }
}

//----------------------------------------------
void CompulsorySleepMode(void) {
  unsigned int Result1;
  Lat0_Hi();
  for (int i = 0; i < 14; i++) {
    Dck0_Hi();
    Dck0_Lo();
  }
  for (int i = 0; i < 12; i++) {
    Dat0_Hi();
    Dat0_Lo();
  }
  Lat0_Lo();
}
//----------------------------------------------
void DemoCompulsorySleepMode(void) {


  if (!digitalRead(CompulsorySleepModeKey1)) {
    int buf1[ICDataBits], buf2[ICDataBits];
    for (int i = 0; i < 16; i++) {
      buf1[i] = buf2[i] = 0;
    }
    LEDOutput(buf1, buf2);
    CompulsorySleepMode();
    delay001();
    while (!digitalRead(CompulsorySleepModeKey1)) {}  //renew240614
    while (digitalRead(CompulsorySleepModeKey1)) {}
    Lat0_Hi();
    Lat0_Lo();
  }
}
//----------------------------------------------
int RextAndGckDetection(void) {
  unsigned int Result1;

  Lat0_Hi();
  for (int i = 0; i < 14; i++) {
    Dck0_Hi();
    Dck0_Lo();
  }
  for (int i = 0; i < 14; i++) {
    Dat0_Hi();
    Dat0_Lo();
  }
  Lat0_Lo();
  Lat0_Lo();
  Result1 = 0;
  if (digitalRead(Dat2)) { bitSet(Result1, 31); }
  for (int i = 31; i > 0; i--) {
    Dck0_Hi();
    if (digitalRead(Dat2)) { bitSet(Result1, (i - 1)); }
    Dck0_Lo();
  }
  Lat0_Lo();
  return Result1;
}
//----------------------------------------------
void DemoREXTPinDetection(void) {
  int buf1[ICDataBits], buf2[ICDataBits];
  int ErrorState1;
  if (!digitalRead(CompulsoryREXTPinDetectionKey1)) {
    delay001();
    while (!digitalRead(CompulsoryREXTPinDetectionKey1)) {}  //renew240614
    while (digitalRead(CompulsoryREXTPinDetectionKey1)) {
      ErrorState1 = RextAndGckDetection();
      for (int i = 0; i < 16; i++) {
        if (bitRead(ErrorState1, 15 - i)) buf1[i] = 0;
        else buf1[i] = DisplayBrightness;
        buf2[i] = 0;
      }
      LEDOutput(buf1, buf2);
      delay002();
      for (int i = 0; i < 16; i++) {
        buf1[i] = 0;
        buf2[i] = 0;
      }
      LEDOutput(buf1, buf2);
      delay002();
    }
  }
}
// Test the battery voltage and display it with DCB code----------------------------------------------
void ReadVinVoltage(void) {
  int buf1[ICDataBits], buf2[ICDataBits];
  int TD1, SD1, FDP1, SDP1, BCD1;
  int VinSensorValue = 0;
  VinSensorValue = analogRead(VinSensorPin);
  VinSensorValue *= 29.9;
  VinSensorValue += 160;

  while (VinSensorValue > 5000 & VinSensorValue < 7000) {
    VinSensorValue = analogRead(VinSensorPin);
    VinSensorValue *= 29.9;
    VinSensorValue += 160;
    TD1 = VinSensorValue / 10000 % 10;  //Ten digit
    SD1 = VinSensorValue / 1000 % 10;   //Single digit
    FDP1 = VinSensorValue / 100 % 10;   //First decimal place
    SDP1 = VinSensorValue / 10 % 10;    //Second decimal place
    BCD1 = (TD1 << 12) + (SD1 << 8) + (FDP1 << 4) + SDP1;

    for (int i = 0; i < 16; i++) {
      if (bitRead(BCD1, 15 - i)) buf1[i] = DisplayBrightness;
      else buf1[i] = 0;
      buf2[i] = 0;
    }
    LEDOutput(buf1, buf2);
    delay(500);
    for (int i = 0; i < 16; i++) { buf1[i] = 0; }
    LEDOutput(buf1, buf2);
    delay(500);
  }
}
//LED gradually dimming display (Fading in / Fading Out)----------------------------------------------
void show1(void) {

  int buf1[ICDataBits], buf2[ICDataBits];

  for (int i = 0; i < 100; i++) {
    for (int j = 0; j < 16; j++) {
      buf1[j] = 0;
      buf2[j] = i * 600;
    }
    LEDOutput(buf1, buf2);
  }

  for (int i = 100; i > 1; i--) {
    for (int j = 0; j < 16; j++) {
      buf1[j] = 0;
      buf2[j] = (i - 1) * 600;
    }
    LEDOutput(buf1, buf2);
  }
}
// Meteor effect light show---------------------------------------------
void show2(void) {

  int buf1[ICDataBits], buf2[ICDataBits];

  for (int i = 0; i < 16; i++) {
    buf1[i] = 0;
    buf2[i] = 0;
  }
  buf2[15] = 60000;
  LEDOutput(buf1, buf2);
  delay(TimeDelay1);

  buf2[14] = 60000;
  buf2[15] = 24000;
  LEDOutput(buf1, buf2);
  delay(TimeDelay1);

  buf2[13] = 60000;
  buf2[14] = 24000;
  buf2[15] = 9200;
  LEDOutput(buf1, buf2);
  delay(TimeDelay1);

  buf2[12] = 60000;
  buf2[13] = 24000;
  buf2[14] = 9200;
  buf2[15] = 3600;
  LEDOutput(buf1, buf2);
  delay(TimeDelay1);

  buf2[11] = 60000;
  buf2[12] = 24000;
  buf2[13] = 9200;
  buf2[14] = 3600;
  buf2[15] = 1400;
  LEDOutput(buf1, buf2);
  delay(TimeDelay1);

  for (int i = 0; i < 16; i++) {
    buf2[i] = 0;
  }
  buf2[10] = 60000;
  buf2[11] = 24000;
  buf2[12] = 9200;
  buf2[13] = 3600;
  buf2[14] = 1400;
  LEDOutput(buf1, buf2);
  delay(TimeDelay1);

  for (int i = 0; i < 16; i++) {
    buf2[i] = 0;
  }
  buf2[9] = 60000;
  buf2[10] = 24000;
  buf2[11] = 9200;
  buf2[12] = 3600;
  buf2[13] = 1400;
  LEDOutput(buf1, buf2);
  delay(TimeDelay1);

  for (int i = 0; i < 16; i++) {
    buf2[i] = 0;
  }
  buf2[8] = 60000;
  buf2[9] = 24000;
  buf2[10] = 9200;
  buf2[11] = 3600;
  buf2[12] = 1400;
  LEDOutput(buf1, buf2);
  delay(TimeDelay1);

  for (int i = 0; i < 16; i++) {
    buf2[i] = 0;
  }
  buf2[7] = 60000;
  buf2[8] = 24000;
  buf2[9] = 9200;
  buf2[10] = 3600;
  buf2[11] = 1400;
  LEDOutput(buf1, buf2);
  delay(TimeDelay1);

  for (int i = 0; i < 16; i++) {
    buf2[i] = 0;
  }
  buf2[6] = 60000;
  buf2[7] = 24000;
  buf2[8] = 9200;
  buf2[9] = 3600;
  buf2[10] = 1400;
  LEDOutput(buf1, buf2);
  delay(TimeDelay1);

  for (int i = 0; i < 16; i++) {
    buf2[i] = 0;
  }
  buf2[5] = 60000;
  buf2[6] = 24000;
  buf2[7] = 9200;
  buf2[8] = 3600;
  buf2[9] = 1400;
  LEDOutput(buf1, buf2);
  delay(TimeDelay1);

  for (int i = 0; i < 16; i++) {
    buf2[i] = 0;
  }
  buf2[4] = 60000;
  buf2[5] = 24000;
  buf2[6] = 9200;
  buf2[7] = 3600;
  buf2[8] = 1400;
  LEDOutput(buf1, buf2);
  delay(TimeDelay1);

  for (int i = 0; i < 16; i++) {
    buf2[i] = 0;
  }
  buf2[3] = 60000;
  buf2[4] = 24000;
  buf2[5] = 9200;
  buf2[6] = 3600;
  buf2[7] = 1400;
  LEDOutput(buf1, buf2);
  delay(TimeDelay1);

  for (int i = 0; i < 16; i++) {
    buf2[i] = 0;
  }
  buf2[2] = 60000;
  buf2[3] = 24000;
  buf2[4] = 9200;
  buf2[5] = 3600;
  buf2[6] = 1400;
  LEDOutput(buf1, buf2);
  delay(TimeDelay1);

  for (int i = 0; i < 16; i++) {
    buf2[i] = 0;
  }
  buf2[1] = 60000;
  buf2[2] = 24000;
  buf2[3] = 9200;
  buf2[4] = 3600;
  buf2[5] = 1400;
  LEDOutput(buf1, buf2);
  delay(TimeDelay1);

  for (int i = 0; i < 16; i++) {
    buf2[i] = 0;
  }
  buf2[0] = 60000;
  buf2[1] = 24000;
  buf2[2] = 9200;
  buf2[3] = 3600;
  buf2[4] = 1400;
  LEDOutput(buf1, buf2);
  delay(TimeDelay1);

  for (int i = 0; i < 16; i++) {
    buf2[i] = 0;
  }
  buf2[0] = 24000;
  buf2[1] = 9200;
  buf2[2] = 3600;
  buf2[3] = 1400;
  LEDOutput(buf1, buf2);
  delay(TimeDelay1);

  for (int i = 0; i < 16; i++) {
    buf2[i] = 0;
  }
  buf2[0] = 9200;
  buf2[1] = 3600;
  buf2[2] = 1400;
  LEDOutput(buf1, buf2);
  delay(TimeDelay1);

  for (int i = 0; i < 16; i++) {
    buf2[i] = 0;
  }
  buf2[0] = 3600;
  buf2[1] = 1400;
  LEDOutput(buf1, buf2);
  delay(TimeDelay1);

  for (int i = 0; i < 16; i++) {
    buf2[i] = 0;
  }
  buf2[0] = 1400;
  LEDOutput(buf1, buf2);
  delay(TimeDelay1);

  for (int i = 0; i < 16; i++) {
    buf2[i] = 0;
  }
  buf2[0] = 60000;
  LEDOutput(buf1, buf2);
  delay(TimeDelay1);

  for (int i = 0; i < 16; i++) {
    buf2[i] = 0;
  }
  buf2[1] = 60000;
  buf2[0] = 39000;
  LEDOutput(buf1, buf2);
  delay(TimeDelay1);

  for (int i = 0; i < 16; i++) {
    buf2[i] = 0;
  }
  buf2[2] = 60000;
  buf2[1] = 39000;
  buf2[0] = 24000;
  LEDOutput(buf1, buf2);
  delay(TimeDelay1);

  for (int i = 0; i < 16; i++) {
    buf2[i] = 0;
  }
  buf2[2] = 60000;
  buf2[1] = 39000;
  buf2[0] = 24000;
  LEDOutput(buf1, buf2);
  delay(TimeDelay1);

  for (int i = 0; i < 16; i++) {
    buf2[i] = 0;
  }
  buf2[3] = 60000;
  buf2[2] = 39000;
  buf2[1] = 24000;
  buf2[0] = 9200;
  LEDOutput(buf1, buf2);
  delay(TimeDelay1);

  for (int i = 0; i < 16; i++) {
    buf2[i] = 0;
  }
  buf2[4] = 60000;
  buf2[3] = 39000;
  buf2[2] = 24000;
  buf2[1] = 9200;
  buf2[0] = 3600;
  LEDOutput(buf1, buf2);
  delay(TimeDelay1);

  for (int i = 0; i < 16; i++) {
    buf2[i] = 0;
  }
  buf2[5] = 60000;
  buf2[4] = 39000;
  buf2[3] = 24000;
  buf2[2] = 9200;
  buf2[1] = 3600;
  buf2[0] = 1400;
  LEDOutput(buf1, buf2);
  delay(TimeDelay1);

  for (int i = 0; i < 16; i++) {
    buf2[i] = 0;
  }
  buf2[6] = 60000;
  buf2[5] = 39000;
  buf2[3] = 24000;
  buf2[2] = 9200;
  buf2[2] = 3600;
  buf2[1] = 1400;
  LEDOutput(buf1, buf2);
  delay(TimeDelay1);

  for (int i = 0; i < 16; i++) {
    buf2[i] = 0;
  }
  buf2[7] = 60000;
  buf2[6] = 39000;
  buf2[5] = 24000;
  buf2[4] = 9200;
  buf2[3] = 3600;
  buf2[2] = 1400;
  LEDOutput(buf1, buf2);
  delay(TimeDelay1);

  for (int i = 0; i < 16; i++) {
    buf2[i] = 0;
  }
  buf2[8] = 60000;
  buf2[7] = 39000;
  buf2[6] = 24000;
  buf2[5] = 9200;
  buf2[4] = 3600;
  buf2[3] = 1400;
  LEDOutput(buf1, buf2);
  delay(TimeDelay1);

  for (int i = 0; i < 16; i++) {
    buf2[i] = 0;
  }
  buf2[9] = 60000;
  buf2[8] = 39000;
  buf2[7] = 24000;
  buf2[6] = 9200;
  buf2[5] = 3600;
  buf2[4] = 1400;
  LEDOutput(buf1, buf2);
  delay(TimeDelay1);

  for (int i = 0; i < 16; i++) {
    buf2[i] = 0;
  }
  buf2[10] = 60000;
  buf2[9] = 39000;
  buf2[8] = 24000;
  buf2[7] = 9200;
  buf2[6] = 3600;
  buf2[5] = 1400;
  LEDOutput(buf1, buf2);
  delay(TimeDelay1);

  for (int i = 0; i < 16; i++) {
    buf2[i] = 0;
  }
  buf2[11] = 60000;
  buf2[10] = 39000;
  buf2[9] = 24000;
  buf2[8] = 9200;
  buf2[7] = 3600;
  buf2[6] = 1400;
  LEDOutput(buf1, buf2);
  delay(TimeDelay1);

  for (int i = 0; i < 16; i++) {
    buf2[i] = 0;
  }
  buf2[12] = 60000;
  buf2[11] = 39000;
  buf2[10] = 24000;
  buf2[9] = 9200;
  buf2[8] = 3600;
  buf2[7] = 1400;
  LEDOutput(buf1, buf2);
  delay(TimeDelay1);

  for (int i = 0; i < 16; i++) {
    buf2[i] = 0;
  }
  buf2[13] = 60000;
  buf2[12] = 39000;
  buf2[11] = 24000;
  buf2[10] = 9200;
  buf2[9] = 3600;
  buf2[8] = 1400;
  LEDOutput(buf1, buf2);
  delay(TimeDelay1);

  for (int i = 0; i < 16; i++) {
    buf2[i] = 0;
  }
  buf2[14] = 60000;
  buf2[13] = 39000;
  buf2[12] = 24000;
  buf2[11] = 9200;
  buf2[10] = 3600;
  buf2[9] = 1400;
  LEDOutput(buf1, buf2);
  delay(TimeDelay1);

  for (int i = 0; i < 16; i++) {
    buf2[i] = 0;
  }
  buf2[15] = 60000;
  buf2[14] = 39000;
  buf2[13] = 24000;
  buf2[12] = 9200;
  buf2[11] = 3600;
  buf2[10] = 1400;
  LEDOutput(buf1, buf2);
  delay(TimeDelay1);

  for (int i = 0; i < 16; i++) {
    buf2[i] = 0;
  }
  buf2[15] = 39000;
  buf2[14] = 24000;
  buf2[13] = 9200;
  buf2[12] = 3600;
  buf2[11] = 1400;
  LEDOutput(buf1, buf2);
  delay(TimeDelay1);

  for (int i = 0; i < 16; i++) {
    buf2[i] = 0;
  }
  buf2[15] = 24000;
  buf2[14] = 9200;
  buf2[13] = 3600;
  buf2[12] = 1400;
  LEDOutput(buf1, buf2);
  delay(TimeDelay1);

  for (int i = 0; i < 16; i++) {
    buf2[i] = 0;
  }
  buf2[15] = 9200;
  buf2[14] = 3600;
  buf2[13] = 1400;
  LEDOutput(buf1, buf2);
  delay(TimeDelay1);

  for (int i = 0; i < 16; i++) {
    buf2[i] = 0;
  }
  buf2[15] = 3600;
  buf2[14] = 1400;
  LEDOutput(buf1, buf2);
  delay(TimeDelay1);

  for (int i = 0; i < 16; i++) {
    buf2[i] = 0;
  }
  buf2[15] = 1400;
  LEDOutput(buf1, buf2);
  delay(TimeDelay1);

  for (int i = 0; i < 16; i++) {
    buf2[i] = 0;
  }
  LEDOutput(buf1, buf2);
}

void PressKeyConfirmation(void) {
  DemoCompulsorySleepMode();
  DemoCompulsoryLEDSmartDetection();
  DemoCompulsoryLEDShortDetection();
  DemoCompulsoryLEDOpenDetection();
  DemoREXTPinDetection();
}
//----------------------------------------------
void loop() {

  CleanCMDBuffer();
  InitICInnrGCK();
  CleanDataBuffer();
  //Test1();
  while (1) {
    //ReadVinVoltage();
    show1();
    PressKeyConfirmation();
    show2();
    PressKeyConfirmation();
  }
}
